@@ -17,14 +17,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+#include <string.h>
 #include "gtktreeselection.h"
 #include "gtktreeprivate.h"
 #include "gtkrbtree.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
-
-static void gtk_tree_selection_init              (GtkTreeSelection      *selection);
-static void gtk_tree_selection_class_init        (GtkTreeSelectionClass *class);
+#include "gtkintl.h"
+#include "gtkalias.h"
 
 static void gtk_tree_selection_finalize          (GObject               *object);
 static gint gtk_tree_selection_real_select_all   (GtkTreeSelection      *selection);
@@ -40,34 +40,9 @@ enum
   LAST_SIGNAL
 };
 
-static GObjectClass *parent_class = NULL;
 static guint tree_selection_signals [LAST_SIGNAL] = { 0 };
 
-GtkType
-gtk_tree_selection_get_type (void)
-{
-  static GtkType selection_type = 0;
-
-  if (!selection_type)
-    {
-      static const GTypeInfo selection_info =
-      {
-        sizeof (GtkTreeSelectionClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-        (GClassInitFunc) gtk_tree_selection_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-        sizeof (GtkTreeSelection),
-	0,              /* n_preallocs */
-        (GInstanceInitFunc) gtk_tree_selection_init
-      };
-
-      selection_type = g_type_register_static (G_TYPE_OBJECT, "GtkTreeSelection", &selection_info, 0);
-    }
-
-  return selection_type;
-}
+G_DEFINE_TYPE (GtkTreeSelection, gtk_tree_selection, G_TYPE_OBJECT)
 
 static void
 gtk_tree_selection_class_init (GtkTreeSelectionClass *class)
@@ -75,18 +50,18 @@ gtk_tree_selection_class_init (GtkTreeSelectionClass *class)
   GObjectClass *object_class;
 
   object_class = (GObjectClass*) class;
-  parent_class = g_type_class_peek_parent (class);
 
   object_class->finalize = gtk_tree_selection_finalize;
   class->changed = NULL;
 
   tree_selection_signals[CHANGED] =
-    gtk_signal_new ("changed",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkTreeSelectionClass, changed),
-		    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new (I_("changed"),
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkTreeSelectionClass, changed),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 }
 
 static void
@@ -109,14 +84,14 @@ gtk_tree_selection_finalize (GObject *object)
     }
 
   /* chain parent_class' handler */
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gtk_tree_selection_parent_class)->finalize (object);
 }
 
 /**
  * _gtk_tree_selection_new:
  *
  * Creates a new #GtkTreeSelection object.  This function should not be invoked,
- * as each #GtkTreeView will create it's own #GtkTreeSelection.
+ * as each #GtkTreeView will create its own #GtkTreeSelection.
  *
  * Return value: A newly created #GtkTreeSelection object.
  **/
@@ -125,7 +100,7 @@ _gtk_tree_selection_new (void)
 {
   GtkTreeSelection *selection;
 
-  selection = GTK_TREE_SELECTION (g_object_new (GTK_TYPE_TREE_SELECTION, NULL));
+  selection = g_object_new (GTK_TYPE_TREE_SELECTION, NULL);
 
   return selection;
 }
@@ -135,7 +110,7 @@ _gtk_tree_selection_new (void)
  * @tree_view: The #GtkTreeView.
  *
  * Creates a new #GtkTreeSelection object.  This function should not be invoked,
- * as each #GtkTreeView will create it's own #GtkTreeSelection.
+ * as each #GtkTreeView will create its own #GtkTreeSelection.
  *
  * Return value: A newly created #GtkTreeSelection object.
  **/
@@ -239,7 +214,7 @@ gtk_tree_selection_set_mode (GtkTreeSelection *selection,
 						  node,
 						  tree,
 						  anchor_path,
-						  0,
+                                                  0,
 						  FALSE);
       if (anchor_path)
 	gtk_tree_path_free (anchor_path);
@@ -334,7 +309,7 @@ gtk_tree_selection_get_tree_view (GtkTreeSelection *selection)
 /**
  * gtk_tree_selection_get_selected:
  * @selection: A #GtkTreeSelection.
- * @model: A pointer set to the #GtkTreeModel, or NULL.
+ * @model: A pointer to set to the #GtkTreeModel, or NULL.
  * @iter: The #GtkTreeIter, or NULL.
  *
  * Sets @iter to the currently selected node if @selection is set to
@@ -359,7 +334,10 @@ gtk_tree_selection_get_selected (GtkTreeSelection  *selection,
   g_return_val_if_fail (GTK_IS_TREE_SELECTION (selection), FALSE);
   g_return_val_if_fail (selection->type != GTK_SELECTION_MULTIPLE, FALSE);
   g_return_val_if_fail (selection->tree_view != NULL, FALSE);
-  g_return_val_if_fail (selection->tree_view->priv->model != NULL, FALSE);
+
+  /* Clear the iter */
+  if (iter)
+    memset (iter, 0, sizeof (GtkTreeIter));
 
   if (model)
     *model = selection->tree_view->priv->model;
@@ -403,13 +381,198 @@ gtk_tree_selection_get_selected (GtkTreeSelection  *selection,
   return retval;
 }
 
+/**
+ * gtk_tree_selection_get_selected_rows:
+ * @selection: A #GtkTreeSelection.
+ * @model: A pointer to set to the #GtkTreeModel, or NULL.
+ *
+ * Creates a list of path of all selected rows. Additionally, if you are
+ * planning on modifying the model after calling this function, you may
+ * want to convert the returned list into a list of #GtkTreeRowReference<!-- -->s.
+ * To do this, you can use gtk_tree_row_reference_new().
+ *
+ * To free the return value, use:
+ * <informalexample><programlisting>
+ * g_list_foreach (list, gtk_tree_path_free, NULL);
+ * g_list_free (list);
+ * </programlisting></informalexample>
+ *
+ * Return value: A #GList containing a #GtkTreePath for each selected row.
+ *
+ * Since: 2.2
+ **/
+GList *
+gtk_tree_selection_get_selected_rows (GtkTreeSelection   *selection,
+                                      GtkTreeModel      **model)
+{
+  GList *list = NULL;
+  GtkRBTree *tree = NULL;
+  GtkRBNode *node = NULL;
+  GtkTreePath *path;
+
+  g_return_val_if_fail (GTK_IS_TREE_SELECTION (selection), NULL);
+  g_return_val_if_fail (selection->tree_view != NULL, NULL);
+
+  if (selection->tree_view->priv->tree == NULL ||
+      selection->tree_view->priv->tree->root == NULL)
+    return NULL;
+
+  if (model)
+    *model = selection->tree_view->priv->model;
+
+  if (selection->type == GTK_SELECTION_NONE)
+    return NULL;
+  else if (selection->type != GTK_SELECTION_MULTIPLE)
+    {
+      GtkTreeIter iter;
+
+      if (gtk_tree_selection_get_selected (selection, NULL, &iter))
+        {
+	  GtkTreePath *path;
+
+	  path = gtk_tree_model_get_path (selection->tree_view->priv->model, &iter);
+	  list = g_list_append (list, path);
+
+	  return list;
+	}
+
+      return NULL;
+    }
+
+  tree = selection->tree_view->priv->tree;
+  node = selection->tree_view->priv->tree->root;
+
+  while (node->left != tree->nil)
+    node = node->left;
+  path = gtk_tree_path_new_first ();
+
+  do
+    {
+      if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
+	list = g_list_prepend (list, gtk_tree_path_copy (path));
+
+      if (node->children)
+        {
+	  tree = node->children;
+	  node = tree->root;
+
+	  while (node->left != tree->nil)
+	    node = node->left;
+
+	  gtk_tree_path_append_index (path, 0);
+	}
+      else
+        {
+	  gboolean done = FALSE;
+
+	  do
+	    {
+	      node = _gtk_rbtree_next (tree, node);
+	      if (node != NULL)
+	        {
+		  done = TRUE;
+		  gtk_tree_path_next (path);
+		}
+	      else
+	        {
+		  node = tree->parent_node;
+		  tree = tree->parent_tree;
+
+		  if (!tree)
+		    {
+		      gtk_tree_path_free (path);
+
+		      goto done; 
+		    }
+
+		  gtk_tree_path_up (path);
+		}
+	    }
+	  while (!done);
+	}
+    }
+  while (TRUE);
+
+  gtk_tree_path_free (path);
+
+ done:
+  return g_list_reverse (list);
+}
+
+static void
+gtk_tree_selection_count_selected_rows_helper (GtkRBTree *tree,
+					       GtkRBNode *node,
+					       gpointer   data)
+{
+  gint *count = (gint *)data;
+
+  if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
+    (*count)++;
+
+  if (node->children)
+    _gtk_rbtree_traverse (node->children, node->children->root,
+			  G_PRE_ORDER,
+			  gtk_tree_selection_count_selected_rows_helper, data);
+}
+
+/**
+ * gtk_tree_selection_count_selected_rows:
+ * @selection: A #GtkTreeSelection.
+ *
+ * Returns the number of rows that have been selected in @tree.
+ *
+ * Return value: The number of rows selected.
+ * 
+ * Since: 2.2
+ **/
+gint
+gtk_tree_selection_count_selected_rows (GtkTreeSelection *selection)
+{
+  gint count = 0;
+
+  g_return_val_if_fail (GTK_IS_TREE_SELECTION (selection), 0);
+  g_return_val_if_fail (selection->tree_view != NULL, 0);
+
+  if (selection->tree_view->priv->tree == NULL ||
+      selection->tree_view->priv->tree->root == NULL)
+    return 0;
+
+  if (selection->type == GTK_SELECTION_SINGLE ||
+      selection->type == GTK_SELECTION_BROWSE)
+    {
+      if (gtk_tree_selection_get_selected (selection, NULL, NULL))
+	return 1;
+      else
+	return 0;
+    }
+
+  _gtk_rbtree_traverse (selection->tree_view->priv->tree,
+                        selection->tree_view->priv->tree->root,
+			G_PRE_ORDER,
+			gtk_tree_selection_count_selected_rows_helper,
+			&count);
+
+  return count;
+}
+
+/* gtk_tree_selection_selected_foreach helper */
+static void
+model_changed (gpointer data)
+{
+  gboolean *stop = (gboolean *)data;
+
+  *stop = TRUE;
+}
+
 /**
  * gtk_tree_selection_selected_foreach:
  * @selection: A #GtkTreeSelection.
  * @func: The function to call for each selected node.
  * @data: user data to pass to the function.
  *
- * Calls a function for each selected node.
+ * Calls a function for each selected node. Note that you cannot modify
+ * the tree or selection from within this function. As a result,
+ * gtk_tree_selection_get_selected_rows() might be more useful.
  **/
 void
 gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
@@ -420,10 +583,13 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
   GtkRBTree *tree;
   GtkRBNode *node;
   GtkTreeIter iter;
+  GtkTreeModel *model;
+
+  gulong inserted_id, deleted_id, reordered_id, changed_id;
+  gboolean stop = FALSE;
 
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
-  g_return_if_fail (selection->tree_view->priv->model != NULL);
 
   if (func == NULL ||
       selection->tree_view->priv->tree == NULL ||
@@ -449,71 +615,96 @@ gtk_tree_selection_selected_foreach (GtkTreeSelection            *selection,
   while (node->left != tree->nil)
     node = node->left;
 
+  model = selection->tree_view->priv->model;
+  g_object_ref (model);
+
+  /* connect to signals to monitor changes in treemodel */
+  inserted_id = g_signal_connect_swapped (model, "row_inserted",
+					  G_CALLBACK (model_changed),
+				          &stop);
+  deleted_id = g_signal_connect_swapped (model, "row_deleted",
+					 G_CALLBACK (model_changed),
+				         &stop);
+  reordered_id = g_signal_connect_swapped (model, "rows_reordered",
+					   G_CALLBACK (model_changed),
+				           &stop);
+  changed_id = g_signal_connect_swapped (selection->tree_view, "notify::model",
+					 G_CALLBACK (model_changed), 
+					 &stop);
+
   /* find the node internally */
   path = gtk_tree_path_new_first ();
-  gtk_tree_model_get_iter (selection->tree_view->priv->model,
-			   &iter, path);
 
   do
     {
       if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED))
-	(* func) (selection->tree_view->priv->model, path, &iter, data);
+        {
+          gtk_tree_model_get_iter (model, &iter, path);
+	  (* func) (model, path, &iter, data);
+        }
+
+      if (stop)
+	goto out;
+
       if (node->children)
 	{
-	  gboolean has_child;
-	  GtkTreeIter tmp;
-
 	  tree = node->children;
 	  node = tree->root;
+
 	  while (node->left != tree->nil)
 	    node = node->left;
-	  tmp = iter;
-	  has_child = gtk_tree_model_iter_children (selection->tree_view->priv->model, &iter, &tmp);
+
 	  gtk_tree_path_append_index (path, 0);
-	  /* Sanity Check! */
-	  TREE_VIEW_INTERNAL_ASSERT_VOID (has_child);
 	}
       else
 	{
 	  gboolean done = FALSE;
+
 	  do
 	    {
 	      node = _gtk_rbtree_next (tree, node);
 	      if (node != NULL)
 		{
-		  gboolean has_next;
-
-		  has_next = gtk_tree_model_iter_next (selection->tree_view->priv->model, &iter);
 		  done = TRUE;
 		  gtk_tree_path_next (path);
-
-		  /* Sanity Check! */
-		  TREE_VIEW_INTERNAL_ASSERT_VOID (has_next);
 		}
 	      else
 		{
-		  gboolean has_parent;
-		  GtkTreeIter tmp_iter = iter;
-
 		  node = tree->parent_node;
 		  tree = tree->parent_tree;
+
 		  if (tree == NULL)
 		    {
-		      gtk_tree_path_free (path);
 		      /* we've run out of tree */
 		      /* We're done with this function */
-		      return;
+
+		      goto out;
 		    }
-		  has_parent = gtk_tree_model_iter_parent (selection->tree_view->priv->model, &iter, &tmp_iter);
+
 		  gtk_tree_path_up (path);
-		  /* Sanity check */
-		  TREE_VIEW_INTERNAL_ASSERT_VOID (has_parent);
 		}
 	    }
 	  while (!done);
 	}
     }
   while (TRUE);
+
+out:
+  if (path)
+    gtk_tree_path_free (path);
+
+  g_signal_handler_disconnect (model, inserted_id);
+  g_signal_handler_disconnect (model, deleted_id);
+  g_signal_handler_disconnect (model, reordered_id);
+  g_signal_handler_disconnect (selection->tree_view, changed_id);
+  g_object_unref (model);
+
+  /* check if we have to spew a scary message */
+  if (stop)
+    g_warning ("The model has been modified from within gtk_tree_selection_selected_foreach.\n"
+	       "This function is for observing the selections of the tree only.  If\n"
+	       "you are trying to get all selected items from the tree, try using\n"
+	       "gtk_tree_selection_get_selected_rows instead.\n");
 }
 
 /**
@@ -529,8 +720,8 @@ gtk_tree_selection_select_path (GtkTreeSelection *selection,
 {
   GtkRBNode *node;
   GtkRBTree *tree;
-  GdkModifierType state = 0;
   gboolean ret;
+  GtkTreeSelectMode mode = 0;
 
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
@@ -546,13 +737,13 @@ gtk_tree_selection_select_path (GtkTreeSelection *selection,
     return;
 
   if (selection->type == GTK_SELECTION_MULTIPLE)
-    state = GDK_CONTROL_MASK;
+    mode = GTK_TREE_SELECT_MODE_TOGGLE;
 
   _gtk_tree_selection_internal_select_node (selection,
 					    node,
 					    tree,
 					    path,
-					    state,
+                                            mode,
 					    FALSE);
 }
 
@@ -588,7 +779,7 @@ gtk_tree_selection_unselect_path (GtkTreeSelection *selection,
 					    node,
 					    tree,
 					    path,
-					    GDK_CONTROL_MASK,
+                                            GTK_TREE_SELECT_MODE_TOGGLE,
 					    TRUE);
 }
 
@@ -670,7 +861,9 @@ gtk_tree_selection_path_is_selected (GtkTreeSelection *selection,
   g_return_val_if_fail (GTK_IS_TREE_SELECTION (selection), FALSE);
   g_return_val_if_fail (path != NULL, FALSE);
   g_return_val_if_fail (selection->tree_view != NULL, FALSE);
-  g_return_val_if_fail (selection->tree_view->priv->model != NULL, FALSE);
+
+  if (selection->tree_view->priv->model == NULL)
+    return FALSE;
 
   ret = _gtk_tree_view_find_node (selection->tree_view,
 				  path,
@@ -689,7 +882,7 @@ gtk_tree_selection_path_is_selected (GtkTreeSelection *selection,
  * @selection: A #GtkTreeSelection
  * @iter: A valid #GtkTreeIter
  * 
- * Returns %TRUE if the row pointed to by @path is currently selected.
+ * Returns %TRUE if the row at @iter is currently selected.
  * 
  * Return value: %TRUE, if @iter is selected
  **/
@@ -776,7 +969,7 @@ gtk_tree_selection_real_select_all (GtkTreeSelection *selection)
  * gtk_tree_selection_select_all:
  * @selection: A #GtkTreeSelection.
  *
- * Selects all the nodes.  @selection is must be set to #GTK_SELECTION_MULTIPLE
+ * Selects all the nodes. @selection must be set to #GTK_SELECTION_MULTIPLE
  * mode.
  **/
 void
@@ -784,12 +977,14 @@ gtk_tree_selection_select_all (GtkTreeSelection *selection)
 {
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
-  if (selection->tree_view->priv->tree == NULL)
+
+  if (selection->tree_view->priv->tree == NULL || selection->tree_view->priv->model == NULL)
     return;
+
   g_return_if_fail (selection->type == GTK_SELECTION_MULTIPLE);
 
   if (gtk_tree_selection_real_select_all (selection))
-    g_signal_emit (G_OBJECT (selection), tree_selection_signals[CHANGED], 0);
+    g_signal_emit (selection, tree_selection_signals[CHANGED], 0);
 }
 
 static void
@@ -885,23 +1080,29 @@ gtk_tree_selection_unselect_all (GtkTreeSelection *selection)
 {
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
-  if (selection->tree_view->priv->tree == NULL)
+
+  if (selection->tree_view->priv->tree == NULL || selection->tree_view->priv->model == NULL)
     return;
   
-  if (selection->tree_view->priv->tree == NULL)
-    return;
-
   if (gtk_tree_selection_real_unselect_all (selection))
-    g_signal_emit (G_OBJECT (selection), tree_selection_signals[CHANGED], 0);
+    g_signal_emit (selection, tree_selection_signals[CHANGED], 0);
 }
 
+enum
+{
+  RANGE_SELECT,
+  RANGE_UNSELECT
+};
+
 static gint
-gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
+gtk_tree_selection_real_modify_range (GtkTreeSelection *selection,
+                                      gint              mode,
 				      GtkTreePath      *start_path,
 				      GtkTreePath      *end_path)
 {
   GtkRBNode *start_node, *end_node;
   GtkRBTree *start_tree, *end_tree;
+  GtkTreePath *anchor_path = NULL;
   gboolean dirty = FALSE;
 
   switch (gtk_tree_path_compare (start_path, end_path))
@@ -915,6 +1116,7 @@ gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
 				start_path,
 				&end_tree,
 				&end_node);
+      anchor_path = start_path;
       break;
     case 0:
       _gtk_tree_view_find_node (selection->tree_view,
@@ -923,6 +1125,7 @@ gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
 				&start_node);
       end_tree = start_tree;
       end_node = start_node;
+      anchor_path = start_path;
       break;
     case -1:
       _gtk_tree_view_find_node (selection->tree_view,
@@ -933,15 +1136,27 @@ gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
 				end_path,
 				&end_tree,
 				&end_node);
+      anchor_path = start_path;
       break;
     }
 
   g_return_val_if_fail (start_node != NULL, FALSE);
   g_return_val_if_fail (end_node != NULL, FALSE);
 
+  if (anchor_path)
+    {
+      if (selection->tree_view->priv->anchor)
+	gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
+
+      selection->tree_view->priv->anchor =
+	gtk_tree_row_reference_new_proxy (G_OBJECT (selection->tree_view),
+	                                  selection->tree_view->priv->model,
+					  anchor_path);
+    }
+
   do
     {
-      dirty |= gtk_tree_selection_real_select_node (selection, start_tree, start_node, TRUE);
+      dirty |= gtk_tree_selection_real_select_node (selection, start_tree, start_node, (mode == RANGE_SELECT)?TRUE:FALSE);
 
       if (start_node == end_node)
 	break;
@@ -976,6 +1191,7 @@ gtk_tree_selection_real_select_range (GtkTreeSelection *selection,
  * @end_path: The final node of the range.
  *
  * Selects a range of nodes, determined by @start_path and @end_path inclusive.
+ * @selection must be set to #GTK_SELECTION_MULTIPLE mode. 
  **/
 void
 gtk_tree_selection_select_range (GtkTreeSelection *selection,
@@ -984,15 +1200,70 @@ gtk_tree_selection_select_range (GtkTreeSelection *selection,
 {
   g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
   g_return_if_fail (selection->tree_view != NULL);
+  g_return_if_fail (selection->type == GTK_SELECTION_MULTIPLE);
+  g_return_if_fail (selection->tree_view->priv->model != NULL);
 
-  if (gtk_tree_selection_real_select_range (selection, start_path, end_path))
-    g_signal_emit (G_OBJECT (selection), tree_selection_signals[CHANGED], 0);
+  if (gtk_tree_selection_real_modify_range (selection, RANGE_SELECT, start_path, end_path))
+    g_signal_emit (selection, tree_selection_signals[CHANGED], 0);
 }
 
+/**
+ * gtk_tree_selection_unselect_range:
+ * @selection: A #GtkTreeSelection.
+ * @start_path: The initial node of the range.
+ * @end_path: The initial node of the range.
+ *
+ * Unselects a range of nodes, determined by @start_path and @end_path
+ * inclusive.
+ *
+ * Since: 2.2
+ **/
+void
+gtk_tree_selection_unselect_range (GtkTreeSelection *selection,
+                                   GtkTreePath      *start_path,
+				   GtkTreePath      *end_path)
+{
+  g_return_if_fail (GTK_IS_TREE_SELECTION (selection));
+  g_return_if_fail (selection->tree_view != NULL);
+  g_return_if_fail (selection->tree_view->priv->model != NULL);
+
+  if (gtk_tree_selection_real_modify_range (selection, RANGE_UNSELECT, start_path, end_path))
+    g_signal_emit (selection, tree_selection_signals[CHANGED], 0);
+}
+
+gboolean
+_gtk_tree_selection_row_is_selectable (GtkTreeSelection *selection,
+				       GtkRBNode        *node,
+				       GtkTreePath      *path)
+{
+  GtkTreeIter iter;
+  gboolean sensitive = FALSE;
+
+  if (!gtk_tree_model_get_iter (selection->tree_view->priv->model, &iter, path))
+    sensitive = TRUE;
+
+  if (!sensitive && selection->tree_view->priv->row_separator_func)
+    {
+      /* never allow separators to be selected */
+      if ((* selection->tree_view->priv->row_separator_func) (selection->tree_view->priv->model,
+							      &iter,
+							      selection->tree_view->priv->row_separator_data))
+	return FALSE;
+    }
+
+  if (selection->user_func)
+    return (*selection->user_func) (selection, selection->tree_view->priv->model, path,
+				    GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED),
+				    selection->user_data);
+  else
+    return TRUE;
+}
+
+
 /* Called internally by gtktreeview.c It handles actually selecting the tree.
  */
 
-/**
+/*
  * docs about the 'override_browse_mode', we set this flag when we want to
  * unset select the node and override the select browse mode behaviour (that is
  * 'one node should *always* be selected').
@@ -1002,7 +1273,7 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
 					  GtkRBNode        *node,
 					  GtkRBTree        *tree,
 					  GtkTreePath      *path,
-					  GdkModifierType   state,
+                                          GtkTreeSelectMode mode,
 					  gboolean          override_browse_mode)
 {
   gint flags;
@@ -1027,7 +1298,7 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
       else if (selection->type == GTK_SELECTION_SINGLE &&
 	       anchor_path && gtk_tree_path_compare (path, anchor_path) == 0)
 	{
-	  if ((state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
+	  if ((mode & GTK_TREE_SELECT_MODE_TOGGLE) == GTK_TREE_SELECT_MODE_TOGGLE)
 	    {
 	      dirty = gtk_tree_selection_real_unselect_all (selection);
 	    }
@@ -1038,17 +1309,7 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
 	    {
 	      /* We only want to select the new node if we can unselect the old one,
 	       * and we can select the new one. */
-	      if (selection->user_func)
-		{
-		  if ((*selection->user_func) (selection, selection->tree_view->priv->model, path,
-					       GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED),
-					       selection->user_data))
-		    dirty = TRUE;
-		}
-	      else
-		{
-		  dirty = TRUE;
-		}
+	      dirty = _gtk_tree_selection_row_is_selectable (selection, node, path);
 
 	      /* if dirty is FALSE, we weren't able to select the new one, otherwise, we try to
 	       * unselect the new one
@@ -1061,7 +1322,11 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
 	      if (dirty)
 		{
 		  if (selection->tree_view->priv->anchor)
-		    gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
+                    {
+                      gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
+                      selection->tree_view->priv->anchor = NULL;
+                    }
+
 		  if (gtk_tree_selection_real_select_node (selection, tree, node, TRUE))
 		    {
 		      selection->tree_view->priv->anchor =
@@ -1074,6 +1339,9 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
 	      if (gtk_tree_selection_real_select_node (selection, tree, node, TRUE))
 		{
 		  dirty = TRUE;
+		  if (selection->tree_view->priv->anchor)
+		    gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
+
 		  selection->tree_view->priv->anchor =
 		    gtk_tree_row_reference_new_proxy (G_OBJECT (selection->tree_view), selection->tree_view->priv->model, path);
 		}
@@ -1082,7 +1350,8 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
     }
   else if (selection->type == GTK_SELECTION_MULTIPLE)
     {
-      if (((state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK) && (anchor_path == NULL))
+      if ((mode & GTK_TREE_SELECT_MODE_EXTEND) == GTK_TREE_SELECT_MODE_EXTEND
+          && (anchor_path == NULL))
 	{
 	  if (selection->tree_view->priv->anchor)
 	    gtk_tree_row_reference_free (selection->tree_view->priv->anchor);
@@ -1091,13 +1360,13 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
 	    gtk_tree_row_reference_new_proxy (G_OBJECT (selection->tree_view), selection->tree_view->priv->model, path);
 	  dirty = gtk_tree_selection_real_select_node (selection, tree, node, TRUE);
 	}
-      else if ((state & (GDK_CONTROL_MASK|GDK_SHIFT_MASK)) == (GDK_SHIFT_MASK|GDK_CONTROL_MASK))
+      else if ((mode & (GTK_TREE_SELECT_MODE_EXTEND | GTK_TREE_SELECT_MODE_TOGGLE)) == (GTK_TREE_SELECT_MODE_EXTEND | GTK_TREE_SELECT_MODE_TOGGLE))
 	{
 	  gtk_tree_selection_select_range (selection,
 					   anchor_path,
 					   path);
 	}
-      else if ((state & GDK_CONTROL_MASK) == GDK_CONTROL_MASK)
+      else if ((mode & GTK_TREE_SELECT_MODE_TOGGLE) == GTK_TREE_SELECT_MODE_TOGGLE)
 	{
 	  flags = node->flags;
 	  if (selection->tree_view->priv->anchor)
@@ -1111,10 +1380,11 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
 	  else
 	    dirty |= gtk_tree_selection_real_select_node (selection, tree, node, TRUE);
 	}
-      else if ((state & GDK_SHIFT_MASK) == GDK_SHIFT_MASK)
+      else if ((mode & GTK_TREE_SELECT_MODE_EXTEND) == GTK_TREE_SELECT_MODE_EXTEND)
 	{
 	  dirty = gtk_tree_selection_real_unselect_all (selection);
-	  dirty |= gtk_tree_selection_real_select_range (selection,
+	  dirty |= gtk_tree_selection_real_modify_range (selection,
+                                                         RANGE_SELECT,
 							 anchor_path,
 							 path);
 	}
@@ -1136,7 +1406,14 @@ _gtk_tree_selection_internal_select_node (GtkTreeSelection *selection,
     gtk_tree_path_free (anchor_path);
 
   if (dirty)
-    g_signal_emit (G_OBJECT (selection), tree_selection_signals[CHANGED], 0);
+    g_signal_emit (selection, tree_selection_signals[CHANGED], 0);  
+}
+
+
+void 
+_gtk_tree_selection_emit_changed (GtkTreeSelection *selection)
+{
+  g_signal_emit (selection, tree_selection_signals[CHANGED], 0);  
 }
 
 /* NOTE: Any {un,}selection ever done _MUST_ be done through this function!
@@ -1148,7 +1425,7 @@ gtk_tree_selection_real_select_node (GtkTreeSelection *selection,
 				     GtkRBNode        *node,
 				     gboolean          select)
 {
-  gboolean selected = FALSE;
+  gboolean toggle = FALSE;
   GtkTreePath *path = NULL;
 
   select = !! select;
@@ -1156,19 +1433,11 @@ gtk_tree_selection_real_select_node (GtkTreeSelection *selection,
   if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED) != select)
     {
       path = _gtk_tree_view_find_path (selection->tree_view, tree, node);
-      if (selection->user_func)
-	{
-	  if ((*selection->user_func) (selection, selection->tree_view->priv->model, path,
-                                       GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_IS_SELECTED),
-                                       selection->user_data))
-	    selected = TRUE;
-	}
-      else
-	selected = TRUE;
+      toggle = _gtk_tree_selection_row_is_selectable (selection, node, path);
       gtk_tree_path_free (path);
     }
 
-  if (selected == TRUE)
+  if (toggle)
     {
       node->flags ^= GTK_RBNODE_IS_SELECTED;
 
@@ -1180,3 +1449,5 @@ gtk_tree_selection_real_select_node (GtkTreeSelection *selection,
   return FALSE;
 }
 
+#define __GTK_TREE_SELECTION_C__
+#include "gtkaliasdef.c"
